﻿namespace CommonUtilityInfrastructure.Tests
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Linq;
    using System.Text;
    using CommonUtilityInfrastructure;
    using NUnit.Framework;
    using SoftwareApproach.TestingExtensions;
    using UsefulTools.CheckboxedTree;
    using UsefulTools.ExtensionMethods;
    using Switch = UsefulTools.Switches.Switch;

    [TestFixture]
    public class TreeTests
    {
        

        private CheckedNode CreateTree()
        {
            var o = new CheckedNode("o");
            var a = new CheckedNode("a");
            var ab = new CheckedNode("ab");
            var aba = new CheckedNode("aba", false);
            var b = new CheckedNode("b");
            var ba = new CheckedNode("ba");
            var baa = new CheckedNode("baa", false);
            var bab = new CheckedNode("bab", false);
            var bac = new CheckedNode("bac");
            var baca = new CheckedNode("baca", false);

            o.Children.AddRange(new[] {a, b});
            a.Children.AddRange(new[] {ab});
            ab.Children.AddRange(new[] {aba});
            b.Children.AddRange(new[] {ba});
            ba.Children.AddRange(new[] {baa, bab, bac});
            bac.Children.AddRange(new[] {baca});

            o.IsIncluded = true;
            b.IsIncluded = false;

            return o;
        }

       

        [Test, ExpectedException(typeof(InvalidOperationException))]
        public void Tree_does_not_allow_adding_self_as_child()
        {
            var a = new CheckedNode("a");
            var ab = new CheckedNode("ab");

            ab.Parent = a;

            a.Children.Add(a);

        }
        [Test]
        public void Tree_allows_valid_child_and_parent()
        {
  
            var a = new CheckedNode("a");
            var ab = new CheckedNode("ab");

            ab.Parent = a;

            a.Children.Add(ab);

            Assert.Pass();
        }
        [Test,ExpectedException(typeof(InvalidOperationException))]
        public void Tree_does_not_allow_incomplete_settings()
        {
            var o = new CheckedNode("o");
            var a = new CheckedNode("a");
            var ab = new CheckedNode("ab");

            ab.Parent = a;

            o.Children.Add(a);
           
        }

        [Test, ExpectedException(typeof(InvalidOperationException))]
        public void Tree_does_not_allow_setting_parent_too_late()
        {
            var o = new CheckedNode("o");
            var a = new CheckedNode("a");
            var ab = new CheckedNode("ab");

            o.Children.Add(a);

            ab.Parent = a;
        }

        [Test, ExpectedException(typeof(InvalidOperationException))]
        public void Tree_does_not_allow_setting_parent_to_self()
        {
            var ab = new CheckedNode("ab");

            ab.Parent = ab;
        }
    }
}

