﻿namespace UsefulTools.Switches
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    public class Switch<T>
    {
        public Switch(T o)
        {
            Value = o;
        }

        public T Value
        {
            get;
            private set;
        }

        public bool HasResult
        {
            get;
            set;
        }
    }

    public static class SwitchMixin
    {
        public static void Default<T>(this Switch<T> @switch, Action action)
        {
            if (!@switch.HasResult)
            {
                action();
            }
        }
        public static void Default<T>(this Switch<T> @switch, Action<T> action)
        {
            if (!@switch.HasResult)
            {
                action(@switch.Value);
            }
        }
        public static void DefaultNop<T>(this Switch<T> @switch)
        {
           
        }
        public static void ThrowIfNoMatch<T>(this Switch<T> @switch)
        {
            if (!@switch.HasResult)
            {
                throw new InvalidOperationException("No case matched value: " + @switch.Value);
            }
        }

        public static Switch<T> Case<T>(this Switch<T> @switch, T caseValue, Action action)
        {
            if (@switch.Value.Equals(caseValue) && !@switch.HasResult)
            {
                action();
                @switch.HasResult = true;
            }
            return @switch;
        }
        public static Switch<T> Case<T>(this Switch<T> @switch, T caseValue1, T caseValue2, Action action)
        {
            if ((@switch.Value.Equals(caseValue1) || @switch.Value.Equals(caseValue2)) && !@switch.HasResult) 
            {
                action();
                @switch.HasResult = true;
            }
            return @switch;
        }
        public static Switch<T> Case<T>(this Switch<T> @switch, T caseValue1, T caseValue2, T caseValue3, Action action)
        {
            if ((@switch.Value.Equals(caseValue1) 
                || @switch.Value.Equals(caseValue2)
                || @switch.Value.Equals(caseValue3)) && !@switch.HasResult)
            {
                action();
                @switch.HasResult = true;
            }
            return @switch;
        }
        public static Switch<T> Case<T>(this Switch<T> @switch, IEnumerable<T> caseValues, Action action)
        {
            if (!@switch.HasResult && caseValues.Contains(@switch.Value) )
            {
                action();
                @switch.HasResult = true;
            }
            return @switch;
        }


        public static Switch<T> CaseNop<T>(this Switch<T> @switch, T caseValue)
        {
            if (@switch.Value.Equals(caseValue))
            {
                @switch.HasResult = true;
            }
            return @switch;
        }
        public static Switch<T> CaseNop<T>(this Switch<T> @switch, T caseValue1, T caseValue2)
        {
            if (@switch.Value.Equals(caseValue1) || @switch.Value.Equals(caseValue2))
            {
                @switch.HasResult = true;
            }
            return @switch;
        }
        public static Switch<T> CaseNop<T>(this Switch<T> @switch, T caseValue1, T caseValue2, T caseValue3)
        {
            if (@switch.Value.Equals(caseValue1)
                || @switch.Value.Equals(caseValue2)
                || @switch.Value.Equals(caseValue3))
            {
                @switch.HasResult = true;
            }
            return @switch;
        }
        public static Switch<T> CaseNop<T>(this Switch<T> @switch, IEnumerable<T> caseValues)
        {
            if (caseValues.Contains(@switch.Value))
            {
                @switch.HasResult = true;
            }
            return @switch;
        }
    }
}