﻿namespace UsefulTools.Switches
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using ExtensionMethods;

    public class ValuedSwitch<T, R>
    {
        public ValuedSwitch(T o)
        {
            Value = o;
        }

        public T Value
        {
            get;
            private set;
        }

        public bool HasResult
        {
            get;
            set;
        }

        public R Result
        {
            get;
            set;
        }
    }

    public static class ValuedSwitchMixin
    {

        public static ValuedSwitch<T, R> Case<T, R>(this ValuedSwitch<T, R> @switch, T caseValue, Func<R> action)
        {
            if (@switch.HasResult)
            {
                if (@switch.Value.Equals(caseValue))
                {
                    throw new InvalidOperationException("Match was already found: " + @switch.Value);
                }
            }
            else if (@switch.Value.Equals(caseValue))
            {
                @switch.Result = action();
                @switch.HasResult = true;
            }

            return @switch;
        }
        public static ValuedSwitch<T, R> Case<T, R>(this ValuedSwitch<T, R> @switch, IEnumerable<T> caseValue, Func<R> action)
        {
            if (@switch.HasResult)
            {
                if (caseValue.Any(v => v.Equals(@switch.Value)))
                {
                    throw new InvalidOperationException("Match was already found: " + @switch.Value);
                }
            }
            else if (caseValue.Any(v => v.Equals(@switch.Value)))
            {
                @switch.Result = action();
                @switch.HasResult = true;
            }

            return @switch;
        }

        public static ValuedSwitch<T, R> Case<T, R>(this ValuedSwitch<T, R> @switch, T caseValue, R result)
        {
            return @switch.Case(caseValue, () => result);
        }
        public static ValuedSwitch<T, R> Case<T, R>(this ValuedSwitch<T, R> @switch, T caseValue, T caseValue2, R result)
        {
            return @switch.Case(new []{caseValue,caseValue2}, () => result);
        }
        public static ValuedSwitch<T, R> Case<T, R>(this ValuedSwitch<T, R> @switch, T caseValue, T caseValue2, T caseValue3, R result)
        {
            return @switch.Case(new[] { caseValue, caseValue2, caseValue3 }, () => result);
        }
        public static ValuedSwitch<T, R> Case<T, R>(this ValuedSwitch<T, R> @switch, IEnumerable<T> caseValues, R result)
        {
            return @switch.Case(caseValues, () => result);
        }
        public static R GetResult<T, R>(this ValuedSwitch<T, R> @switch)
        {
            if (!@switch.HasResult)
            {
                throw new InvalidOperationException("No case matched value: " + @switch.Value);
            }
            return @switch.Result;
        }
        public static R Default<T, R>(this ValuedSwitch<T, R> @switch, R result)
        {
            if (!@switch.HasResult)
            {
                return result;
            }
            else
            {
                return @switch.Result;
            }
        }
        public static R Default<T, R>(this ValuedSwitch<T, R> @switch, Func<R> action)
        {
            if (!@switch.HasResult)
            {
                return action();
            }
            else
            {
                return @switch.Result;
            }
        }
        public static R Default<T, R>(this ValuedSwitch<T, R> @switch, Func<T, R> action)
        {
            if (!@switch.HasResult)
            {
                return action(@switch.Value);
            }
            else
            {
                return @switch.Result;
            }
        }
    }
}